package OpenERP::OOM::Tutorial;


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

OpenERP::OOM::Tutorial

=head1 VERSION

version 0.43

=head1 DESCRIPTION

=over 4

=item L<OpenERP::OOM::Tutorial::Schema> - Setting up an OpenERP::OOM Schema

=item L<OpenERP::OOM::Tutorial::Searching> - Tips on searching

=back

=head1 NAME

OpenERP::OOM::Tutorial

=head1 COPYRIGHT & LICENSE

Copyright (C) 2012 OpusVL

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.

=head1 AUTHOR

Jon Allen (JJ), <jj@opusvl.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2011-2016 by OpusVL.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
